/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 and 2000 by the Free Software Foundation.      *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <stdio.h>

#include "ttsgf.h"
#include "liberty.h"


/* 
 * Define all global variables used within the engine.
 */


/* The go board and position. */
Intersection p[MAX_BOARD][MAX_BOARD];
int          ko_i;
int          ko_j;

int color_has_played;

/* Hashing of positions. */
Hashtable  * movehash;
Hash_data    hashdata;
/* 
 * A study showed that this is the optimal setting for when hashing only 
 * a few functions.  Hashing all functions saves time, but wastes table
 * space.  This is bad when the reading is complicated.  This is a 
 * compromise.
 */
int hashflags = HASH_DEFAULT;

int potential_moves[MAX_BOARD][MAX_BOARD];

int  black_captured;
int  white_captured;    /* number of black and white stones captured */

/* Used by reading. */
int stackp;             /* stack pointer */
int movenum;            /* movenumber */
int depth;              /* deep reading cut off */
int backfill_depth;     /* deep reading cut off */
int fourlib_depth;      /* deep reading cut off */
int ko_depth;
int nominal_depth;          /* deep reading cut off, nominal value */
int nominal_backfill_depth; /* deep reading cut off, nominal value */
int nominal_fourlib_depth;  /* deep reading cut off, nominal value */
int nominal_ko_depth;

/* Miscellaneous. */
int showstack        = 0; /* debug stack pointer */
int showstatistics   = 0; /* print statistics */
int profile_patterns = 0; /* print statistics of pattern usage */
int allpats          = 0; /* generate all patterns, even small ones */
int printworms       = 0; /* print full data on each string */
int printmoyo        = 0; /* print moyo board each move */
int printboard       = 0; /* print board each move */
int board_size       = 19;/* board size */
int count_variations = 0; /* used by decide_string */
int sgf_dump         = 0; /* used by decide_string */
int loading          = 0; /* TRUE indicates last loaded move comes from file */
int style  = STY_DEFAULT; /* style of play */
int mouse            = 0; /* use mouse code */
int life             = 0; /* use eye reading code */
int life_eyesize     = 0;
int fusekidb         = 0; /* use fuseki database */
int showtime         = 0; /* print time to find move */
int hurry            = 0; /* normally 0, but 1, 2, or 3 to play faster */
int terse            = 0; /* When printing, be as terse as possible. */
int debug            = 0;
int verbose          = 0; /* trace level */
int analyzerflag     = 0;

/* Various statistics are collected here. */
struct stats_data stats;

struct worm_data      worm[MAX_BOARD][MAX_BOARD];
struct dragon_data    dragon[MAX_BOARD][MAX_BOARD];
struct half_eye_data  half_eye[MAX_BOARD][MAX_BOARD];
struct eye_data       black_eye[MAX_BOARD][MAX_BOARD];
struct eye_data       white_eye[MAX_BOARD][MAX_BOARD];

int  distance_to_black[MAX_BOARD][MAX_BOARD];
int  distance_to_white[MAX_BOARD][MAX_BOARD];
int  strategic_distance_to_black[MAX_BOARD][MAX_BOARD];
int  strategic_distance_to_white[MAX_BOARD][MAX_BOARD];
int  black_domain[MAX_BOARD][MAX_BOARD];
int  white_domain[MAX_BOARD][MAX_BOARD];


SGFNode *sgf_root = NULL;
char *analyzerfile = NULL;

