/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 and 2000 by the Free Software Foundation.      *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


/*
 * Location_set: A set of locations on the go board.
 */


#include <stdio.h>

#include "liberty.h"
#include "location_set.h"

/* 
 * Lookup table of bits.  
 * NOTE: We only need MAX_BOARD of these for the current application.
 */

static unsigned long bitpatterns[] = {
  0x00000001, 0x00000002, 0x00000004, 0x00000008, /* 00-03 */
  0x00000010, 0x00000020, 0x00000040, 0x00000080, /* 05-07 */
  0x00000100, 0x00000200, 0x00000400, 0x00000800, /* 08-11 */
  0x00001000, 0x00002000, 0x00004000, 0x00008000, /* 12-13 */
  0x00010000, 0x00020000, 0x00040000, 0x00080000, /* 16-19 */
  0x00100000, 0x00200000, 0x00400000, 0x00800000, /* 20-23 */
  0x01000000, 0x02000000, 0x04000000, 0x08000000, /* 24-27 */
  0x10000000, 0x20000000, 0x40000000, 0x80000000, /* 28-31 */
};

#if 0 /* Unused. /gf */
static unsigned int bitmasks[] = {
  0xfe, 0xfc, 0xf8, 0xf0,
  0xe0, 0xc0, 0x80, 0xff
};


static unsigned int lowest_bit[] = {
 -1, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*   0- 15 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*  16- 31 */
  5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*  32- 47 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*  48- 63 */
  6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*  64- 79 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*  80- 95 */
  5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /*  96-111 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 112-127 */
  7, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 128-143 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 144-159 */
  5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 160-175 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 176-191 */
  6, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 192-207 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 208-223 */
  5, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 224-239 */
  4, 0, 1, 0, 2, 0, 1, 0, 3, 0, 1, 0, 2, 0, 1, 0, /* 240-255 */
};
#endif

/* ================================================================ */
/*                     Location_set proper                          */
/* ================================================================ */


/*
 * Clear a location_set.
 */

void
location_set_clear(Location_set *ls)
{
  int  i;

  for (i = 0; i < board_size; ++i)
    ls->bitmap[i] = 0;
}


/*
 * Add location (m, n) to the location_set ls.
 */

void
location_set_add(Location_set *ls, int m, int n)
{
  ls->bitmap[m] |= bitpatterns[n];
}


/*
 * Remove location (m, n) from the location_set ls.
 */

void
location_set_del(Location_set *ls, int m, int n)
{
  ls->bitmap[m] &= ~bitpatterns[n];
}


/*
 * Print the Location_set ls to the file outfile.
 */

void
location_set_print(FILE *outfile, Location_set *ls)
{
  Location_set_iterator  iter;
  int                    m, n;

  fprintf(outfile, "Set: ");
  for (ls_iter_init(ls, &iter);
       ls_iter_next(&iter, &m, &n);
       ls_iter_update(&iter))
    {
      /*fprintf(outfile, "[%d %d] ", iter.current_bitmap, iter.current_bit);*/
      fprintf(outfile, "(%d,%d)", m, n);
    }
  fprintf(outfile, "\n");
}


/*
 * Print the Location_set ls to the file outfile using a primitive format.
 */

void
location_set_dump(FILE *outfile, Location_set *ls)
{
  int  i, j;
  int  bit;

  for (i = 0; i < board_size; i++) {
    fprintf(outfile, "%2d ", i);
    bit = 1;
    for (j = 0; j < board_size; j++) {
      fprintf(outfile, (ls->bitmap[i] & bit) ? "1" : "0");
      bit <<= 1;
    }
    fprintf(outfile, "\n");
  }
}


/*
 * Copy a location_set from another one.
 */

void
location_set_copy(Location_set *to, Location_set *from)
{
  *to = *from;
}


/*
 * Expand the bits of the location set one bit in all directions.
 * Limit it to board_size.  If the parameter diagonnaly is true,
 * the expansion will be in all 8 directions instead of 4.
 */

void
location_set_expand(Location_set *ls, int diagonally)
{
  Location_set  copy;
  int  i;

  /* Keep a copy of the initial set. We us it later when we
   * expand horizontally to avoid that everything expands
   * diagonally as well. 
   */
  if (!diagonally)
    location_set_copy(&copy, ls);

  /* Expand upwards. */
  for (i = 0; i < board_size - 1; ++i)
    ls->bitmap[i] |= ls->bitmap[i+1];

  /* Expand downwards. */
  for (i = board_size - 1; i > 0; --i)
    ls->bitmap[i] |= ls->bitmap[i-1];

  /* Expand left and right and limit it to the board size. */
  if (diagonally)
    for (i = 0; i < board_size; ++i)
      ls->bitmap[i] |= (((ls->bitmap[i] << 1) | (ls->bitmap[i] >> 1))
		      & ~bitpatterns[board_size]);
  else
    for (i = 0; i < board_size; ++i)
      ls->bitmap[i] |= (((copy.bitmap[i] << 1) | (copy.bitmap[i] >> 1))
		      & ~bitpatterns[board_size]);
}


/*
 * Set ls to the union between this set and the other.
 */

void
location_set_union(Location_set *ls, Location_set *other)
{
  int  i;

  for (i = 0; i < board_size; ++i)
    ls->bitmap[i] |= other->bitmap[i];
}


/*
 *Set ls to the intersection between this set and the other.
 */

void
location_set_intersection(Location_set *ls, Location_set *other)
{
  int  i;

  for (i = 0; i < board_size; ++i)
    ls->bitmap[i] &= other->bitmap[i];
}


/*
 * Subtract the other set from this one.
 */

void
location_set_subtract(Location_set *ls, Location_set *other)
{
  int  i;

  for (i = 0; i < board_size; ++i)
    ls->bitmap[i] &= ~(other->bitmap[i]);
}


/* ================================================================ */
/*                  Iterator for Location_set                       */
/* ================================================================ */


/* 
 * Usage:
 *
 * Location_set            set;
 * Location_set_iterator   iter;
 * int                     x, y;
 *
 * for (ls_iter_init(&set, &iter);
 *      ls_iter_next(&iter, &x, &y);
 *      ls_iter_update(&iter))
 *   use_it(x, y);
 */

void
ls_iter_init(Location_set *ls, Location_set_iterator *lsi)
{
  lsi->ls = ls;
  lsi->current_bitmap = 0;
  lsi->current_bit = -1;

  ls_iter_update(lsi);
}


int
ls_iter_next(Location_set_iterator *lsi, int *m, int *n)
{
  /* If we have passed all usable bitmaps, we are finished. */
  if (lsi->current_bitmap == board_size)
    return 0;

  *m = lsi->current_bitmap;
  *n = lsi->current_bit;

  return 1;
}


void
ls_iter_update(Location_set_iterator *lsi)
{
  /* FIXME: This is a somewhat inefficient implementation. 
            A better way might be to split the bitmaps into 
	    bytes and then treat the bytes separately using lookup 
	    tables. */
  do {
    lsi->current_bit++;
    if (lsi->current_bit >= board_size) {
      lsi->current_bit = 0;

      /* Skip the bitmaps that are empty. */
      do {
	lsi->current_bitmap++;
      } while (lsi->current_bitmap < board_size
	       && lsi->ls->bitmap[lsi->current_bitmap] == 0);
    }
  } while (lsi->current_bitmap < board_size
	   && !(lsi->ls->bitmap[lsi->current_bitmap] 
		& bitpatterns[lsi->current_bit]));
}


/*
 * Local Variables:
 * tab-width: 8
 * c-basic-offset: 2
 * End:
 */
