/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 and 2000 by the Free Software Foundation.      *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>

#include "gnugo.h"


#include "interface.h"
#include "main.h"



/* ---------------------------------------------------------------
 * the game option structure
 * --------------------------------------------------------------- */

struct game_options {
  int quiet;
  int display_board;
};

static struct game_options gmopt;
static struct game_options *gopt = &gmopt;


/* ---------------------------------------------------------------
 * These functions deal with move manipulation
 * --------------------------------------------------------------- */


/*
 * Return a move for the given color and return the move value.
 */

int 
get_move(Gameinfo *ginfo, int *x, int *y, int move)
{
  int move_val;

  assert(move == WHITE || move == BLACK);

  /* Generate computer move. */
  move_val = position_genmove(&ginfo->position, x, y, move,
			      ginfo->move_number);
  position_play_move(&ginfo->position, *x, *y, move);

  sgffile_move_made(*x, *y, move, move_val);
  ginfo->move_number++;
  
  if (move_val >= 0)
    /* A move */
    return 1;
  else
    /* a pass */
    return 0;
}


/*
 * Return a test move for the given color and return the move value 
 * This move does not get registered on the board.
 */

int 
get_test_move(Gameinfo *ginfo, int *x, int *y, int move)
{
  int move_val;

  if ((move != WHITE) && (move != BLACK)) {
    fprintf(stderr, "get_test_move: Illegal move color specified: %d\n", move);
    move = ginfo->to_move;
  }

  /* Generate computer move. */
  move_val = position_genmove(&ginfo->position, x, y, move,
			      ginfo->move_number);

  /* A value less than 0 is a pass */
  return move_val >= 0;
}


/*
 * Make a move with legality checking.
 */

int 
put_move(Position *pos, int x, int y, int move)
{
  if (!position_legal(pos, x, y, move)) {
    gprintf("Illegal move: %m\n", x, y);
    return 0;
  }
  
  position_play_move(pos, x, y, move);

  /* Send the move to the file, move_val=0. */
  sgffile_move_made(x, y, move, 0);
  return 1;
}


/*
 * Make a move without legality checking.
 */

int 
put_stone(Position *pos, int x, int y, int color)
{
  position_add_stone(pos, x, y, color);
  sgffile_put_stone(x, y, color);
  return 1;
}


/* ---------------------------------------------------------------
 * These functions deal with the game option structure
 * --------------------------------------------------------------- */


void
init_gopt()
{
  gopt->quiet = 0;
  gopt->display_board = 0;
}


/*
 * Turn off some informational messages.
 */

int 
get_opt_quiet()
{
  return gopt->quiet;
}

void
set_opt_quiet(int num)
{ 
  gopt->quiet = num;
}


/*
 * Turn off board displays.
 */

int 
get_opt_display_board()
{
  return gopt->display_board;
}

void
set_opt_display_board(int num)
{ 
  gopt->display_board = num;
}



/*
 * Local Variables:
 * tab-width: 8
 * c-basic-offset: 2
 * End:
 */
