/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see   *
 * http://www.gnu.org/software/gnugo/ for more information.      *
 *                                                               *
 * Copyright 1999 and 2000 by the Free Software Foundation.      *
 *                                                               *
 * This program is free software; you can redistribute it and/or *
 * modify it under the terms of the GNU General Public License   *
 * as published by the Free Software Foundation - version 2.     *
 *                                                               *
 * This program is distributed in the hope that it will be       *
 * useful, but WITHOUT ANY WARRANTY; without even the implied    *
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR       *
 * PURPOSE.  See the GNU General Public License in file COPYING  *
 * for more details.                                             *
 *                                                               *
 * You should have received a copy of the GNU General Public     *
 * License along with this program; if not, write to the Free    *
 * Software Foundation, Inc., 59 Temple Place - Suite 330,       *
 * Boston, MA 02111, USA                                         *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */





#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "liberty.h"
#include "patterns.h"


#define TRYMOVE(x, y, z) trymove(x, y, z, "helper", -1, -1)
#define OFFSET(x, y, z, w) offset(x, y, ti, tj, &z, &w, transformation)
#define ARGS struct pattern *pattern, int transformation, int ti, int tj, int color


/* This file contains helper functions which assist the pattern matcher.
 * They are invoked with ti,tj = the posn on the board marked with '*'.
 * They are invoked with color = WHITE or BLACK : any pieces on the
 * board marked with 'O' in the pattern will always contain color,
 * and 'X's contain OTHER_COLOR(color)
 *
 * The helper must return 0 if the pattern is rejected and 1 otherwise.
 */




/*

XO     cb
O*     at

Check whether a cut is feasible and effective.

*/

int 
basic_cut_helper (ARGS)
{
  int ai, aj, bi, bj, ci, cj;
  int acolor, ccolor;
  UNUSED(pattern);
  
  OFFSET(0, -1, ai, aj);  /* O to west */
  OFFSET(-1, 0, bi, bj);  /* O to north */
  OFFSET(-1, -1, ci, cj); /* X to northwest */

  acolor=p[ai][aj];
  ccolor=OTHER_COLOR(acolor);

  ASSERT(p[ai][aj]!=EMPTY, ai, aj);
  ASSERT(p[bi][bj]==acolor, bi, bj);
  ASSERT(p[ci][cj]==ccolor, ci, cj);

  /* If c is a ko stone, assume that we would lose the ko. */
  if (worm[ci][cj].attacki != -1
      && (ccolor == color
	  || !worm[ci][cj].ko))
    return 0;
  if (worm[ti][tj].ko)
    return 0;

  if (TRYMOVE(ti, tj, ccolor)) {
    if (attack(ti, tj, NULL, NULL) || attack(ci, cj, NULL, NULL)) {
      popgo();
      return 0;
    }
#if 0
    /* This case should be covered by attack(ci, cj). */
    if (worm[ci][cj].liberties==1) {
      popgo();
      return 0;    /* better to capture (ci,cj) */
    }
#endif
    popgo();
  }
  else
    return 0;

  if (!safe_move(ti, tj, acolor))
    return 0;

  /* Cut ok. */
  return 1;
}


/*

?XO?     ?be?  Take an important ko
X*XO	 atXd
?XO?	 ?cf?

or:

?OX?     ?be?  Connect an important ko
O*OX	 atOd
?OX?	 ?cf?

The ko is deemed important if it unites distinct dragons, unless they
are already all alive. It is deemed still more important if it cuts or
connects dragons for both colors.

Note: This change may have the effect to overvalue certain kos. I
still would like to try it out though, since it's necessary to somehow
increase the value for many early kos. /gf

*/
#if 0
int 
ko_important_helper (ARGS)
{
  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej, fi, fj;
  int tval1=0, tval2=0;
  UNUSED(pattern);

  if (!worm[ti][tj].ko)
    return (0);

  if (!safe_move(ti, tj, color))
    return 0;

  OFFSET( 0, -1, ai, aj);
  OFFSET(-1,  0, bi, bj);
  OFFSET( 1,  0, ci, cj);
  OFFSET( 0,  2, di, dj);
  OFFSET(-1,  1, ei, ej);
  OFFSET( 1,  1, fi, fj);
  ASSERT((p[ai][aj]==p[bi][bj]), bi, bj);
  ASSERT((p[ai][aj]==p[ci][cj]), ci, cj);
  ASSERT((p[di][dj]==p[ei][ej]), ei, ej);
  ASSERT((p[di][dj]==p[fi][fj]), fi, fj);
  if ((dragon[ai][aj].origini != dragon[bi][bj].origini) ||
      (dragon[ai][aj].originj != dragon[bi][bj].originj))
    tval1=connection_value(ai, aj, bi, bj, ti, tj);
  if ((dragon[ai][aj].origini != dragon[ci][cj].origini) ||
      (dragon[ai][aj].originj != dragon[ci][cj].originj))
    tval1=max(tval1, connection_value(ai, aj, ci, cj, ti, tj));
  if ((dragon[di][dj].origini != dragon[ei][ej].origini) ||
      (dragon[di][dj].originj != dragon[ei][ej].originj))
    tval2=connection_value(di, dj, ei, ej, ti, tj);
  if ((dragon[di][dj].origini != dragon[fi][fj].origini) ||
      (dragon[di][dj].originj != dragon[fi][fj].originj))
    tval2=max(tval1, connection_value(di, dj, fi, fj, ti, tj));

/* Since the player who loses the ko gets two outside moves, we should 
   divide the value by 2.
*/

  return min(COMPUTE_SCORE, (tval1+tval2)/2);
}
#endif


/*

?XO?     ?be?  Take an important ko
X*XO	 atXd
----     ----

*/
#if 0
int 
edge_ko_important_helper (ARGS)
{
  int ai, aj, bi, bj, di, dj, ei, ej;
  int tval1=0, tval2=0;
  UNUSED(pattern);

  if (!worm[ti][tj].ko) return (0);

  if (!safe_move(ti, tj, color))
    return (0);

  OFFSET( 0, -1, ai, aj);
  OFFSET(-1,  0, bi, bj);
  OFFSET( 0,  2, di, dj);
  OFFSET(-1,  1, ei, ej);

  ASSERT((p[ai][aj]==p[bi][bj]), bi, bj);
  ASSERT((p[di][dj]==p[ei][ej]), ei, ej);
  if ((dragon[ai][aj].origini != dragon[bi][bj].origini) ||
      (dragon[ai][aj].originj != dragon[bi][bj].originj))
    tval1=connection_value(ai, aj, bi, bj, ti, tj);
  if ((dragon[di][dj].origini != dragon[ei][ej].origini) ||
      (dragon[di][dj].originj != dragon[ei][ej].originj))
    tval2=connection_value(di, dj, ei, ej, ti, tj);
  return min(COMPUTE_SCORE, (tval1+tval2)/2);
}
#endif


/*

*X       tc
XO       bO
O.       Oa
?O       ?O

:8,85,0,0,0,0,-,0,fight_ko_helper

*/

#if 0
int 
fight_ko_helper (ARGS)
{
  int ai, aj, bi, bj, ci, cj;
  int ei, ej;
  int tval=0;
  UNUSED(pattern);

  OFFSET(2, 1, ai, aj);  
  OFFSET(0, 1, bi, bj);  
  OFFSET(1, 0, ci, cj);  

  if (!worm[ai][aj].ko) 
    return (0);
  if (TRYMOVE(ai, aj, color)) {
    if (p[bi][bj] && attack(bi, bj, &ei, &ej)) {
      if ((ei==ti) && (ej==tj))
	tval=worm[bi][bj].value-5;
    }
    if (p[ci][cj] && attack(ci, cj, &ei, &ej)) {
      if ((ei==ti) && (ej==tj))
	tval=max(tval,worm[ci][cj].value-5);
    }
    popgo();
  }
  tval=min(COMPUTE_SCORE,tval);
  return (tval);
}
#endif

/*
      start a ko fight

?OX?     ?Ob?
.XOX	 aXOX
?*.O	 ?tcO
?oOo	 ?oOo

:8,85,0,0,0,0,-,0,fight_ko2_helper

*/
#if 0
int 
fight_ko2_helper (ARGS)
{
  int ai, aj, bi, bj, ci, cj;
  int tval=0;
  int other=OTHER_COLOR(color);
  UNUSED(pattern);

  OFFSET(-1, -1, ai, aj);  
  OFFSET(-2, 1, bi, bj);  
  OFFSET(0, 1, ci, cj);  

  if (TRYMOVE(ti, tj, color)) {
    if (TRYMOVE(ai, aj, other)) {
      if (TRYMOVE(ci, cj, color)) {
	if (p[bi][bj] && attack(bi, bj, NULL, NULL))
	  tval=min(COMPUTE_SCORE,worm[bi][bj].value);
	popgo();
      }
      popgo();
    }
    popgo();
  }
  return (tval);
}
#endif


/* Jump out into nothingness. To avoid jumping into our own territory,
   we don't allow the strategic distance to the opponent to be too
   large. Also we never ever jump into our own established eyespace. */

int 
jump_out_helper (ARGS)
{
  int dist;
  int own_eyespace;

  UNUSED(transformation); UNUSED(pattern);

  if (0)
    TRACE("  jump out to %m --- distance to B=%d; to W=%d\n", ti, tj, 
	  distance_to_black[ti][tj], distance_to_white[ti][tj]);
  if (color==WHITE)
  {
    dist = strategic_distance_to_black[ti][tj];
    own_eyespace = (white_eye[ti][tj].color == WHITE_BORDER);
  }
  else
  {
    dist = strategic_distance_to_white[ti][tj];
    own_eyespace = (black_eye[ti][tj].color == BLACK_BORDER);
  }
  /* This is an experiment to see if we can do without the strategic
   * distance measure.
   */
#if 0  
  if (dist!=-1 && dist<=6 && !own_eyespace)
    return 1;
  else
    return 0;
#else
  if (influence_area_color(ti, tj) != color && !own_eyespace)
    return 1;
  else
    return 0;
#endif
}


/* Make a long jump into nothingness. Since these jumps are not
   securely connected we don't use them to jump into the opponent's
   zone of control. */

int 
jump_out_far_helper (ARGS)
{
  if (influence_area_color(ti, tj) != OTHER_COLOR(color))
    return jump_out_helper(pattern, transformation, ti, tj, color);
  else
    return 0;
}


/*

break double connection 

?X.               ?da  
O*X               ctO
?X.               ?eb

*/

int
double_does_break_helper (ARGS)
{
  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
  int acolor, dcolor;
  int safe=0;
  UNUSED(pattern);
  UNUSED(color);
  
  OFFSET(-1, 1, ai, aj);
  OFFSET(1, 1, bi, bj);
  OFFSET(0, -1, ci, cj);
  OFFSET(-1, 0, di, dj);
  OFFSET(1, 0, ei, ej);
  
  ASSERT(p[ai][aj]==EMPTY, ai, aj);
  ASSERT(p[bi][bj]==EMPTY, bi, bj);
  ASSERT(p[ci][cj]!=EMPTY, ci, cj);

  acolor=p[ci][cj];
  dcolor=OTHER_COLOR(acolor);

  ASSERT(p[di][dj]==dcolor, di, dj);
  ASSERT(p[ei][ej]==dcolor, ei, ej);

  if (TRYMOVE (ti, tj, acolor)) 
    {
      if (TRYMOVE (ai, aj, dcolor)) {
	if (TRYMOVE (bi, bj, acolor)) 
	  {
	    if (!p[ci][cj] ||
		!p[bi][bj] ||
		attack(ci, cj, NULL, NULL) ||
		attack(bi, bj, NULL, NULL))
	      safe = 1;
	    popgo();
	  }
	else 
	  safe = 1;        /* cutting at b is illegal */
	popgo();
      }
      if (TRYMOVE (bi, bj, dcolor)) {           /* defender gets to decide */
	if (TRYMOVE (ai, aj, acolor)) 
	  {
	    if (!p[ci][cj] ||
		!p[ai][aj] ||
		attack(ci, cj, NULL, NULL) ||
		attack(ai, aj, NULL, NULL))
	      safe = 1;
	    popgo();
	  }
	else 
	  safe = 1;       /* cutting at a is illegal */
	popgo();
      }
      popgo();
    }
  else                   
    safe = 1;         /* pushing at a is illegal; */
  return !safe;
}
  

/* Active until the opponent has played his first stone. */

int
high_handicap_helper (ARGS)
{
  int other=OTHER_COLOR(color);
  UNUSED(transformation); UNUSED(pattern); UNUSED(ti); UNUSED(tj);
  
  if(!(color_has_played & other)) {
    return 1;
  }
  return 0;
}


int
wide_break_helper (ARGS)
{
  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
  int does_break = 1;
  int other=OTHER_COLOR(color);
  UNUSED(pattern);
  
  TRACE("in wide_break_helper at %m\n", ti, tj);
  OFFSET(-1, 0, ai, aj);
  OFFSET(-1, 1, bi, bj);
  OFFSET(-1, 2, ci, cj);
  OFFSET(0, 1, di, dj);
  OFFSET(0, 2, ei, ej);

  if (TRYMOVE(ti, tj, color)) {
    if (TRYMOVE(bi, bj, other)) {
      if (TRYMOVE(di, dj, color)) {
	if ((p[ci][cj]==EMPTY) || (attack(ci, cj, NULL, NULL)) ||
	    (p[ti][tj]==EMPTY) || (attack(ti, tj, NULL, NULL)))
	  does_break = 0;
	popgo();
      }
      popgo();
    }
    popgo();
  }

  if (!does_break)
    return 0;
  
  if (TRYMOVE(ti, tj, color)) {
    if (TRYMOVE(di, dj, other)) {
      if (TRYMOVE(bi, bj, color)) {
	if ((p[ci][cj]==EMPTY) || (attack(ci, cj, NULL, NULL)) ||
	    (p[ti][tj]==EMPTY) || (attack(ti, tj, NULL, NULL)))
	  does_break = 0;
	popgo();
      }
      popgo();
    }
    popgo();
  }
  return does_break;
}


/*
  
XXO      XXc            decrease eye space in sente (unless it kills)
.*X      eta
---      ---

or

XXO      XXc            decrease eye space in sente (unless it kills)
.*X      eta
XXO      XXd

or

|XXO     |XXc           decrease eye space in sente (unless it kills)
|.*X     |eta
|XXO     |XXd

or

|XXO     |XXc           decrease eye space in sente (unless it kills)
|.*X     |eta
+---     +---

*/

int
throw_in_atari_helper(ARGS)
{
  int ai, aj, bi, bj, ci, cj, di, dj, ei, ej;
  int success = 0;
  int other = OTHER_COLOR(color);
  int libi[2], libj[2];
  UNUSED(pattern);
  
  OFFSET(0, 1, ai, aj);
  OFFSET(-1, 1, ci, cj);
  OFFSET(1, 1, di, dj);
  OFFSET(0, -1, ei, ej);

  /* Find second liberty of the stone a. */
  findlib(ai, aj, 2, libi, libj);
  if ((libi[0] != ti) || (libj[0] != tj)) {
    bi = libi[0];
    bj = libj[0];
  }
  else {
    bi = libi[1];
    bj = libj[1];
  }
  
  if (TRYMOVE(ti, tj, color)) {
    if (!attack(ci, cj, NULL, NULL) &&
	!(ON_BOARD(di, dj) && attack(di, dj, NULL, NULL))) {
      if (TRYMOVE(bi, bj, other)) {
	if (attack(ai, aj, NULL, NULL))
	  success = 1;
	popgo();
      }
      else {
	success = 1; /* X move at (bi, bj) would have been suicide */
      }
    }
    popgo();
  }

  /* The followup is to capture the "a" string. Estimate the value to
     twice the size. */
  add_followup_value(ti, tj, 2*worm[ai][aj].size);

  return success;
}
#if 0
  /* No more than partial credit if only one stone in atari, as this
   * leaves a potential ko.
   */
  if (worm[ai][aj].size == 1)
    tval -= 20;

  /* Also reduce value if the stone at e might make a difference on
   * the outside.
   */

  if (other==BLACK) 
    {
      if ((black_eye[ei][ej].marginal) ||
	  (black_eye[ei][ej].marginal_neighbors - black_eye[ti][tj].marginal > 0))
	tval -=20;
    }
  else if (white_eye[ei][ej].marginal ||
      (white_eye[ei][ej].marginal_neighbors - white_eye[ti][tj].marginal > 0))
    tval -=20;

  /* Discard move completely if it's unlikely that capturing the
   * stones would make any difference. (Rather crude heuristics.) */
  if (worm[ai][aj].size <= 2) {
    /* Don't consider the edge cases, being nondead is specified in
     * the pattern for those.
     */
    if (ON_BOARD(di, dj))
      if (dragon[ci][cj].status == DEAD && dragon[di][dj].status == DEAD) 
	if (connection_value(ci, cj, di, dj, ti, tj) == 0)
	  tval = 0;
  }
  return tval;
}
#endif


/*

Prevent misreporting of a as lunch for b.
To be used in autohelper action line. E.g.

XO|          ba|
O*|          O*|
oo|          oo|
?o|          ?o|

>not_lunch(a,b);
*/

int
not_lunch_helper(int ai, int aj, int bi, int bj)
{
  int m, n;

  if (worm[ai][aj].size>2)
    return 0;

  /* Tell the move generation code about the change in status. */
  remove_lunch(bi, bj, ai, aj);
  
  if ((dragon[bi][bj].lunchi==ai) && (dragon[bi][bj].lunchj==aj))
    for (m=0; m<board_size; m++)
      for (n=0; n<board_size; n++)
	if ((dragon[m][n].origini==dragon[bi][bj].origini) &&
	    (dragon[m][n].originj==dragon[bi][bj].originj))
	  dragon[m][n].lunchi=-1;
  return 0;
}
  

/*

?OO                 ?OO
O.O                 O.b
.*.                 at.
---                 ---

overturn spurious attack found by find_cap2

*/


int
indirect_helper (ARGS)
{
  int ai, aj, bi, bj;

  OFFSET(0, -1, ai, aj);
  OFFSET(-1, 1, bi, bj);
  UNUSED(pattern);
  UNUSED(color);

  if ((worm[bi][bj].attacki==ti) && (worm[bi][bj].attackj==tj)
      && (worm[bi][bj].defendi == -1) && (does_defend(ai, aj, bi, bj)))
    change_attack(bi, bj, -1, -1, 0);
  return 0;
}


  
/* This is intended for use in autohelpers. */
/* Check whether the string at (ai, aj) can attack any surrounding
 * string. If so, return false as the move to create a seki (probably)
 * wouldn't work.
 */
int 
seki_helper(int ai, int aj)
{
  int r;
  int adj;
  int adji[MAXCHAIN];
  int adjj[MAXCHAIN];
  int adjsize[MAXCHAIN];
  int adjlib[MAXCHAIN];
  
  chainlinks(ai, aj, &adj, adji, adjj, adjsize, adjlib);
  for (r=0; r<adj; r++)
    if (worm[adji[r]][adjj[r]].attacki != -1)
      return 0;
  return 1;
}


/*

XO     aO
O*     O*

Used in a connection pattern to ensure that X is a cutstone.
*/

int 
ugly_cutstone_helper (ARGS)
{
  int ai, aj;
  UNUSED(pattern);
  UNUSED(color);
  
  OFFSET(-1, -1, ai, aj);

  worm[ai][aj].cutstone++;
  return 0;
}


/*

XO     aO
O*     O*

Increase the cutstone2 field if * is a potential cutting point, i.e.  if it
does work as a cutting point once 'a' has been defended. This helper is
expected to always return 0.  

*/

int 
cutstone2_helper (ARGS)
{
  int ai, aj;
  int bi, bj;
  int ci, cj;
  int di, dj;
  UNUSED(pattern);
  UNUSED(color);
  
  OFFSET(-1, -1, ai, aj);
  OFFSET(-1,  0, bi, bj);
  OFFSET( 0, -1, ci, cj);

  if (worm[ai][aj].defendi == -1)
    return 0;
  
  di = worm[ai][aj].defendi;
  dj = worm[ai][aj].defendj;

  if (TRYMOVE(di, dj, p[ai][aj]))
  {
    if (!p[bi][bj] || attack(bi, bj, NULL, NULL)
	|| !p[ci][cj] || attack(ci, cj, NULL, NULL)
	|| safe_move(ti, tj, p[ai][aj])) {
      popgo();
      worm[worm[ai][aj].origini][worm[ai][aj].originj].cutstone2++;
      propagate_worm(worm[ai][aj].origini, worm[ai][aj].originj);
      return 0;
    }
    popgo();
  }
  return 0;
}

/*
 * This is intended for use in autohelpers.
 *
 * Give a conservative estimate of the value of saving the string (ai, aj)
 * by capturing one opponent stone.
 */
void
threaten_to_save_helper(int ti, int tj, int ai, int aj)
{
  add_followup_value(ti, tj, 2 + 2 * worm[ai][aj].effective_size);
}


/*
 * This is intended for use in conn.db autohelpers.
 *
 * Amalgamate either a with b or c with b, depending on which of the
 * two dragons a and c is largest.
 *
 * If either of these pairs already have been amalgamated somehow,
 * do nothing.
 */
void
amalgamate_most_valuable_helper(int ai, int aj, int bi, int bj, int ci, int cj)
{
  if (!same_dragon(ai, aj, bi, bj) && !same_dragon(bi, bj, ci, cj)) {
    if (dragon[ai][aj].effective_size >= dragon[ci][cj].effective_size)
      join_dragons(ai, aj, bi, bj);
    else
      join_dragons(bi, bj, ci, cj);
  }
}

/*
 * LOCAL Variables:
 * tab-width: 8
 * c-basic-offset: 2
 * End:
 */
